@@ -110,6 +110,11 @@ class model {
      */
     protected $target = null;
 
+    /**
+     * @var \core_analytics\predictor
+     */
+    protected $predictionsprocessor = null;
+
     /**
      * @var \core_analytics\local\indicator\base[]
      */
@@ -284,7 +289,7 @@ class model {
                     $timesplitting = \core_analytics\manager::get_time_splitting($options['timesplitting']);
                     $timesplittings = array($timesplitting->get_id() => $timesplitting);
                 } else {
-                    $timesplittings = \core_analytics\manager::get_enabled_time_splitting_methods();
+                    $timesplittings = \core_analytics\manager::get_time_splitting_methods_for_evaluation();
                 }
             } else {
 
@@ -301,12 +306,6 @@ class model {
             }
         }
 
-        if (!empty($options['evaluation'])) {
-            foreach ($timesplittings as $timesplitting) {
-                $timesplitting->set_evaluating(true);
-            }
-        }
-
         $classname = $target->get_analyser_class();
         if (!class_exists($classname)) {
             throw new \coding_exception($classname . ' class does not exists');
@@ -333,14 +332,14 @@ class model {
      *
      * @param \core_analytics\local\target\base $target
      * @param \core_analytics\local\indicator\base[] $indicators
-     * @param string $timesplittingid The time splitting method id (its fully qualified class name)
+     * @param string|false $timesplittingid The time splitting method id (its fully qualified class name)
+     * @param string|null $processor The machine learning backend this model will use.
      * @return \core_analytics\model
      */
-    public static function create(\core_analytics\local\target\base $target, array $indicators, $timesplittingid = false) {
+    public static function create(\core_analytics\local\target\base $target, array $indicators,
+                                  $timesplittingid = false, $processor = null) {
         global $USER, $DB;
 
-        \core_analytics\manager::check_can_manage_models();
-
         $indicatorclasses = self::indicator_classes($indicators);
 
         $now = time();
@@ -353,6 +352,28 @@ class model {
         $modelobj->timemodified = $now;
         $modelobj->usermodified = $USER->id;
 
+        if ($target->based_on_assumptions()) {
+            $modelobj->trained = 1;
+        }
+
+        if ($timesplittingid) {
+            if (!\core_analytics\manager::is_valid($timesplittingid, '\core_analytics\local\time_splitting\base')) {
+                throw new \moodle_exception('errorinvalidtimesplitting', 'analytics');
+            }
+            if (substr($timesplittingid, 0, 1) !== '\\') {
+                throw new \moodle_exception('errorinvalidtimesplitting', 'analytics');
+            }
+            $modelobj->timesplitting = $timesplittingid;
+        }
+
+        if ($processor &&
+                !manager::is_valid($processor, '\core_analytics\classifier') &&
+                !manager::is_valid($processor, '\core_analytics\regressor')) {
+            throw new \coding_exception('The provided predictions processor \\' . $processor . '\processor is not valid');
+        } else {
+            $modelobj->predictionsprocessor = $processor;
+        }
+
         $id = $DB->insert_record('analytics_models', $modelobj);
 
         // Get db defaults.
@@ -360,14 +381,6 @@ class model {
 
         $model = new static($modelobj);
 
-        if ($timesplittingid) {
-            $model->enable($timesplittingid);
-        }
-
-        if ($model->is_static()) {
-            $model->mark_as_trained();
-        }
-
         return $model;
     }
 
@@ -386,6 +399,10 @@ class model {
 
         $existingmodels = $DB->get_records('analytics_models', array('target' => $target->get_id()));
 
+        if (!$existingmodels) {
+            return false;
+        }
+
         if (!$indicators && $existingmodels) {
             return true;
         }
@@ -411,9 +428,10 @@ class model {
      * @param int|bool $enabled
      * @param \core_analytics\local\indicator\base[]|false $indicators False to respect current indicators
      * @param string|false $timesplittingid False to respect current time splitting method
+     * @param string|false $predictionsprocessor False to respect current predictors processor value
      * @return void
      */
-    public function update($enabled, $indicators = false, $timesplittingid = '') {
+    public function update($enabled, $indicators = false, $timesplittingid = '', $predictionsprocessor = false) {
         global $USER, $DB;
 
         \core_analytics\manager::check_can_manage_models();
@@ -433,14 +451,21 @@ class model {
             $timesplittingid = $this->model->timesplitting;
         }
 
+        if ($predictionsprocessor === false) {
+            // Respect current value.
+            $predictionsprocessor = $this->model->predictionsprocessor;
+        }
+
         if ($this->model->timesplitting !== $timesplittingid ||
-                $this->model->indicators !== $indicatorsstr) {
+                $this->model->indicators !== $indicatorsstr ||
+                $this->model->predictionsprocessor !== $predictionsprocessor) {
 
             // Delete generated predictions before changing the model version.
             $this->clear();
 
             // It needs to be reset as the version changes.
             $this->uniqueid = null;
+            $this->indicators = null;
 
             // We update the version of the model so different time splittings are not mixed up.
             $this->model->version = $now;
@@ -458,6 +483,7 @@ class model {
         $this->model->enabled = intval($enabled);
         $this->model->indicators = $indicatorsstr;
         $this->model->timesplitting = $timesplittingid;
+        $this->model->predictionsprocessor = $predictionsprocessor;
         $this->model->timemodified = $now;
         $this->model->usermodified = $USER->id;
 
@@ -477,8 +503,14 @@ class model {
         $this->clear();
 
         // Method self::clear is already clearing the current model version.
-        $predictor = \core_analytics\manager::get_predictions_processor();
-        $predictor->delete_output_dir($this->get_output_dir(array(), true));
+        $predictor = $this->get_predictions_processor(false);
+        if ($predictor->is_ready() !== true) {
+            $predictorname = \core_analytics\manager::get_predictions_processor_name($predictor);
+            debugging('Prediction processor ' . $predictorname . ' is not ready to be used. Model ' .
+                $this->model->id . ' could not be deleted.');
+        } else {
+            $predictor->delete_output_dir($this->get_output_dir(array(), true));
+        }
 
         $DB->delete_records('analytics_models', array('id' => $this->model->id));
         $DB->delete_records('analytics_models_log', array('modelid' => $this->model->id));
@@ -503,10 +535,33 @@ class model {
             $this->get_analyser()->add_log(get_string('noevaluationbasedassumptions', 'analytics'));
             $result = new \stdClass();
             $result->status = self::NO_DATASET;
-            return array($this->get_time_splitting()->get_id() => $result);
+            return array($result);
         }
 
         $options['evaluation'] = true;
+
+        if (empty($options['mode'])) {
+            $options['mode'] = 'configuration';
+        }
+
+        switch ($options['mode']) {
+            case 'trainedmodel':
+
+                // We are only interested on the time splitting method used by the trained model.
+                $options['timesplitting'] = $this->model->timesplitting;
+
+                // Provide the trained model directory to the ML backend if that is what we want to evaluate.
+                $trainedmodeldir = $this->get_output_dir(['execution']);
+                break;
+            case 'configuration':
+
+                $trainedmodeldir = false;
+                break;
+
+            default:
+                throw new \moodle_exception('errorunknownaction', 'analytics');
+        }
+
         $this->init_analyser($options);
 
         if (empty($this->get_indicators())) {
@@ -516,7 +571,7 @@ class model {
         $this->heavy_duty_mode();
 
         // Before get_labelled_data call so we get an early exception if it is not ready.
-        $predictor = \core_analytics\manager::get_predictions_processor();
+        $predictor = $this->get_predictions_processor();
 
         $datasets = $this->get_analyser()->get_labelled_data();
 
@@ -545,10 +600,10 @@ class model {
             // Evaluate the dataset, the deviation we accept in the results depends on the amount of iterations.
             if ($this->get_target()->is_linear()) {
                 $predictorresult = $predictor->evaluate_regression($this->get_unique_id(), self::ACCEPTED_DEVIATION,
-                self::EVALUATION_ITERATIONS, $dataset, $outputdir);
+                    self::EVALUATION_ITERATIONS, $dataset, $outputdir, $trainedmodeldir);
             } else {
                 $predictorresult = $predictor->evaluate_classification($this->get_unique_id(), self::ACCEPTED_DEVIATION,
-                self::EVALUATION_ITERATIONS, $dataset, $outputdir);
+                    self::EVALUATION_ITERATIONS, $dataset, $outputdir, $trainedmodeldir);
             }
 
             $result->status = $predictorresult->status;
@@ -566,7 +621,7 @@ class model {
                 $dir = $predictorresult->dir;
             }
 
-            $result->logid = $this->log_result($timesplitting->get_id(), $result->score, $dir, $result->info);
+            $result->logid = $this->log_result($timesplitting->get_id(), $result->score, $dir, $result->info, $options['mode']);
 
             $results[$timesplitting->get_id()] = $result;
         }
@@ -608,7 +663,7 @@ class model {
         $outputdir = $this->get_output_dir(array('execution'));
 
         // Before get_labelled_data call so we get an early exception if it is not ready.
-        $predictor = \core_analytics\manager::get_predictions_processor();
+        $predictor = $this->get_predictions_processor();
 
         $datasets = $this->get_analyser()->get_labelled_data();
 
@@ -675,50 +730,67 @@ class model {
         // Before get_unlabelled_data call so we get an early exception if it is not writable.
         $outputdir = $this->get_output_dir(array('execution'));
 
-        // Before get_unlabelled_data call so we get an early exception if it is not ready.
         if (!$this->is_static()) {
-            $predictor = \core_analytics\manager::get_predictions_processor();
-        }
+            // Predictions using a machine learning backend.
 
-        $samplesdata = $this->get_analyser()->get_unlabelled_data();
+            // Before get_unlabelled_data call so we get an early exception if it is not ready.
+            $predictor = $this->get_predictions_processor();
 
-        // Get the prediction samples file.
-        if (empty($samplesdata) || empty($samplesdata[$this->model->timesplitting])) {
+            $samplesdata = $this->get_analyser()->get_unlabelled_data();
 
-            $result = new \stdClass();
-            $result->status = self::NO_DATASET;
-            $result->info = $this->get_analyser()->get_logs();
-            return $result;
-        }
-        $samplesfile = $samplesdata[$this->model->timesplitting];
+            // Get the prediction samples file.
+            if (empty($samplesdata) || empty($samplesdata[$this->model->timesplitting])) {
 
-        // We need to throw an exception if we are trying to predict stuff that was already predicted.
-        $params = array('modelid' => $this->model->id, 'action' => 'predicted', 'fileid' => $samplesfile->get_id());
-        if ($predicted = $DB->get_record('analytics_used_files', $params)) {
-            throw new \moodle_exception('erroralreadypredict', 'analytics', '', $samplesfile->get_id());
-        }
+                $result = new \stdClass();
+                $result->status = self::NO_DATASET;
+                $result->info = $this->get_analyser()->get_logs();
+                return $result;
+            }
+            $samplesfile = $samplesdata[$this->model->timesplitting];
 
-        $indicatorcalculations = \core_analytics\dataset_manager::get_structured_data($samplesfile);
+            // We need to throw an exception if we are trying to predict stuff that was already predicted.
+            $params = array('modelid' => $this->model->id, 'action' => 'predicted', 'fileid' => $samplesfile->get_id());
+            if ($predicted = $DB->get_record('analytics_used_files', $params)) {
+                throw new \moodle_exception('erroralreadypredict', 'analytics', '', $samplesfile->get_id());
+            }
 
-        // Prepare the results object.
-        $result = new \stdClass();
+            $indicatorcalculations = \core_analytics\dataset_manager::get_structured_data($samplesfile);
 
-        if ($this->is_static()) {
-            // Prediction based on assumptions.
-            $result->status = self::OK;
-            $result->info = [];
-            $result->predictions = $this->get_static_predictions($indicatorcalculations);
-
-        } else {
             // Estimation and classification processes run on the machine learning backend side.
             if ($this->get_target()->is_linear()) {
                 $predictorresult = $predictor->estimate($this->get_unique_id(), $samplesfile, $outputdir);
             } else {
                 $predictorresult = $predictor->classify($this->get_unique_id(), $samplesfile, $outputdir);
             }
+
+            // Prepare the results object.
+            $result = new \stdClass();
             $result->status = $predictorresult->status;
             $result->info = $predictorresult->info;
             $result->predictions = $this->format_predictor_predictions($predictorresult);
+
+        } else {
+            // Predictions based on assumptions.
+
+            $indicatorcalculations = $this->get_analyser()->get_static_data();
+            // Get the prediction samples file.
+            if (empty($indicatorcalculations) || empty($indicatorcalculations[$this->model->timesplitting])) {
+
+                $result = new \stdClass();
+                $result->status = self::NO_DATASET;
+                $result->info = $this->get_analyser()->get_logs();
+                return $result;
+            }
+
+            // Same as reset($indicatorcalculations) as models based on assumptions only analyse 1 single
+            // time-splitting method.
+            $indicatorcalculations = $indicatorcalculations[$this->model->timesplitting];
+
+            // Prepare the results object.
+            $result = new \stdClass();
+            $result->status = self::OK;
+            $result->info = [];
+            $result->predictions = $this->get_static_predictions($indicatorcalculations);
         }
 
         if ($result->status !== self::OK) {
@@ -726,18 +798,31 @@ class model {
         }
 
         if ($result->predictions) {
-            $samplecontexts = $this->execute_prediction_callbacks($result->predictions, $indicatorcalculations);
+            list($samplecontexts, $predictionrecords) = $this->execute_prediction_callbacks($result->predictions,
+                $indicatorcalculations);
         }
 
         if (!empty($samplecontexts) && $this->uses_insights()) {
-            $this->trigger_insights($samplecontexts);
+            $this->trigger_insights($samplecontexts, $predictionrecords);
         }
 
-        $this->flag_file_as_used($samplesfile, 'predicted');
+        if (!$this->is_static()) {
+            $this->flag_file_as_used($samplesfile, 'predicted');
+        }
 
         return $result;
     }
 
+    /**
+     * Returns the model predictions processor.
+     *
+     * @param bool $checkisready
+     * @return \core_analytics\predictor
+     */
+    public function get_predictions_processor($checkisready = true) {
+        return manager::get_predictions_processor($this->model->predictionsprocessor, $checkisready);
+    }
+
     /**
      * Formats the predictor results.
      *
@@ -757,7 +842,7 @@ class model {
                         // skip it and do nothing with it.
                         debugging($this->model->id . ' model predictions processor could not process the sample with id ' .
                             $sampleinfo[0], DEBUG_DEVELOPER);
-                        continue;
+                        continue 2;
                     case 2:
                         // Prediction processors that do not return a prediction score will have the maximum prediction
                         // score.
@@ -784,7 +869,7 @@ class model {
      * @param array $indicatorcalculations
      * @return array
      */
-    protected function execute_prediction_callbacks($predictions, $indicatorcalculations) {
+    protected function execute_prediction_callbacks(&$predictions, $indicatorcalculations) {
 
         // Here we will store all predictions' contexts, this will be used to limit which users will see those predictions.
         $samplecontexts = array();
@@ -794,7 +879,6 @@ class model {
 
             // The unique sample id contains both the sampleid and the rangeindex.
             list($sampleid, $rangeindex) = $this->get_time_splitting()->infer_sample_info($uniquesampleid);
-
             if ($this->get_target()->triggers_callback($prediction->prediction, $prediction->predictionscore)) {
 
                 // Prepare the record to store the predicted values.
@@ -816,33 +900,44 @@ class model {
             $this->save_predictions($records);
         }
 
-        return $samplecontexts;
+        return [$samplecontexts, $records];
     }
 
     /**
      * Generates insights and updates the cache.
      *
      * @param \context[] $samplecontexts
+     * @param  \stdClass[] $predictionrecords
      * @return void
      */
-    protected function trigger_insights($samplecontexts) {
+    protected function trigger_insights($samplecontexts, $predictionrecords) {
 
         // Notify the target that all predictions have been processed.
-        $this->get_target()->generate_insight_notifications($this->model->id, $samplecontexts);
+        if ($this->get_analyser()::one_sample_per_analysable()) {
 
-        // Update cache.
-        $cache = \cache::make('core', 'contextwithinsights');
-        foreach ($samplecontexts as $context) {
-            $modelids = $cache->get($context->id);
-            if (!$modelids) {
-                // The cache is empty, but we don't know if it is empty because there are no insights
-                // in this context or because cache/s have been purged, we need to be conservative and
-                // "pay" 1 db read to fill up the cache.
-                $models = \core_analytics\manager::get_models_with_insights($context);
-                $cache->set($context->id, array_keys($models));
-            } else if (!in_array($this->get_id(), $modelids)) {
-                array_push($modelids, $this->get_id());
-                $cache->set($context->id, $modelids);
+            // We need to do something unusual here. self::save_predictions uses the bulk-insert function (insert_records()) for
+            // performance reasons and that function does not return us the inserted ids. We need to retrieve them from
+            // the database, and we need to do it using one single database query (for performance reasons as well).
+            $predictionrecords = $this->add_prediction_ids($predictionrecords);
+
+            // Get \core_analytics\prediction objects also fetching the samplesdata. This costs us
+            // 1 db read, but we have to pay it if we want that our insights include links to the
+            // suggested actions.
+            $predictions = array_map(function($predictionobj) {
+                $prediction = new \core_analytics\prediction($predictionobj, $this->prediction_sample_data($predictionobj));
+                return $prediction;
+            }, $predictionrecords);
+        } else {
+            $predictions = [];
+        }
+
+        $this->get_target()->generate_insight_notifications($this->model->id, $samplecontexts, $predictions);
+
+        if ($this->get_target()->link_insights_report()) {
+
+            // Update cache.
+            foreach ($samplecontexts as $context) {
+                \core_analytics\manager::cached_models_with_insights($context, $this->get_id());
             }
         }
     }
@@ -855,12 +950,23 @@ class model {
      */
     protected function get_static_predictions(&$indicatorcalculations) {
 
+        $headers = array_shift($indicatorcalculations);
+
+        // Get rid of the sampleid header.
+        array_shift($headers);
+
         // Group samples by analysable for \core_analytics\local\target::calculate.
         $analysables = array();
         // List all sampleids together.
         $sampleids = array();
 
         foreach ($indicatorcalculations as $uniquesampleid => $indicators) {
+
+            // Get rid of the sampleid column.
+            unset($indicators[0]);
+            $indicators = array_combine($headers, $indicators);
+            $indicatorcalculations[$uniquesampleid] = $indicators;
+
             list($sampleid, $rangeindex) = $this->get_time_splitting()->infer_sample_info($uniquesampleid);
 
             $analysable = $this->get_analyser()->get_sample_analysable($sampleid);
@@ -875,6 +981,7 @@ class model {
                     'sampleids' => array()
                 ];
             }
+
             // Using the sampleid as a key so we can easily merge indicators data later.
             $analysables[$analysableclass][$rangeindex]->indicatorsdata[$sampleid] = $indicators;
             // We could use indicatorsdata keys but the amount of redundant data is not that big and leaves code below cleaner.
@@ -885,7 +992,7 @@ class model {
         }
 
         // Get all samples data.
-        list($sampleids, $samplesdata) = $this->get_analyser()->get_samples($sampleids);
+        list($sampleids, $samplesdata) = $this->get_samples($sampleids);
 
         // Calculate the targets.
         $predictions = array();
@@ -898,16 +1005,21 @@ class model {
                 $this->get_target()->add_sample_data($data->indicatorsdata);
 
                 // Append new elements (we can not get duplicates because sample-analysable relation is N-1).
-                $range = $this->get_time_splitting()->get_range_by_index($rangeindex);
+                $timesplitting = $this->get_time_splitting();
+                $timesplitting->set_modelid($this->get_id());
+                $timesplitting->set_analysable($data->analysable);
+                $range = $timesplitting->get_range_by_index($rangeindex);
+
                 $this->get_target()->filter_out_invalid_samples($data->sampleids, $data->analysable, false);
                 $calculations = $this->get_target()->calculate($data->sampleids, $data->analysable, $range['start'], $range['end']);
 
                 // Missing $indicatorcalculations values in $calculations are caused by is_valid_sample. We need to remove
                 // these $uniquesampleid from $indicatorcalculations because otherwise they will be stored as calculated
                 // by self::save_prediction.
-                $indicatorcalculations = array_filter($indicatorcalculations, function($indicators, $uniquesampleid) use ($calculations) {
-                    list($sampleid, $rangeindex) = $this->get_time_splitting()->infer_sample_info($uniquesampleid);
-                    if (!isset($calculations[$sampleid])) {
+                $indicatorcalculations = array_filter($indicatorcalculations, function($indicators, $uniquesampleid)
+                        use ($calculations, $rangeindex) {
+                    list($sampleid, $indicatorsrangeindex) = $this->get_time_splitting()->infer_sample_info($uniquesampleid);
+                    if ($rangeindex == $indicatorsrangeindex && !isset($calculations[$sampleid])) {
                         return false;
                     }
                     return true;
@@ -957,6 +1069,7 @@ class model {
 
         $analysable = $this->get_analyser()->get_sample_analysable($sampleid);
         $timesplitting = $this->get_time_splitting();
+        $timesplitting->set_modelid($this->get_id());
         $timesplitting->set_analysable($analysable);
         $range = $timesplitting->get_range_by_index($rangeindex);
         if ($range) {
@@ -986,8 +1099,6 @@ class model {
     public function enable($timesplittingid = false) {
         global $DB, $USER;
 
-        \core_analytics\manager::check_can_manage_models();
-
         $now = time();
 
         if ($timesplittingid && $timesplittingid !== $this->model->timesplitting) {
@@ -1195,7 +1306,7 @@ class model {
             return $prediction->sampleid;
         }, $predictions);
 
-        list($unused, $samplesdata) = $this->get_analyser()->get_samples($sampleids);
+        list($unused, $samplesdata) = $this->get_samples($sampleids);
 
         $current = 0;
 
@@ -1237,7 +1348,7 @@ class model {
      */
     public function prediction_sample_data($predictionobj) {
 
-        list($unused, $samplesdata) = $this->get_analyser()->get_samples(array($predictionobj->sampleid));
+        list($unused, $samplesdata) = $this->get_samples(array($predictionobj->sampleid));
 
         if (empty($samplesdata[$predictionobj->sampleid])) {
             throw new \moodle_exception('errorsamplenotavailable', 'analytics');
@@ -1270,7 +1381,7 @@ class model {
      * @param bool $onlymodelid Preference over $subdirs
      * @return string
      */
-    protected function get_output_dir($subdirs = array(), $onlymodelid = false) {
+    public function get_output_dir($subdirs = array(), $onlymodelid = false) {
         global $CFG;
 
         $subdirstr = '';
@@ -1319,16 +1430,21 @@ class model {
     }
 
     /**
-     * Exports the model data.
+     * Exports the model data for displaying it in a template.
      *
+     * @param \renderer_base $output The renderer to use for exporting
      * @return \stdClass
      */
-    public function export() {
+    public function export(\renderer_base $output) {
 
         \core_analytics\manager::check_can_manage_models();
 
         $data = clone $this->model;
+
+        $data->modelname = format_string($this->get_name());
+        $data->name = $this->inplace_editable_name()->export_for_template($output);
         $data->target = $this->get_target()->get_name();
+        $data->targetclass = $this->get_target()->get_id();
 
         if ($timesplitting = $this->get_time_splitting()) {
             $data->timesplitting = $timesplitting->get_name();
@@ -1341,6 +1457,59 @@ class model {
         return $data;
     }
 
+    /**
+     * Exports the model data to a zip file.
+     *
+     * @param string $zipfilename
+     * @param bool $includeweights Include the model weights if available
+     * @return string Zip file path
+     */
+    public function export_model(string $zipfilename, bool $includeweights = true) : string {
+
+        \core_analytics\manager::check_can_manage_models();
+
+        $modelconfig = new model_config($this);
+        return $modelconfig->export($zipfilename, $includeweights);
+    }
+
+    /**
+     * Imports the provided model.
+     *
+     * Note that this method assumes that model_config::check_dependencies has already been called.
+     *
+     * @throws \moodle_exception
+     * @param  string $zipfilepath Zip file path
+     * @return \core_analytics\model
+     */
+    public static function import_model(string $zipfilepath) : \core_analytics\model {
+
+        \core_analytics\manager::check_can_manage_models();
+
+        $modelconfig = new \core_analytics\model_config();
+        return $modelconfig->import($zipfilepath);
+    }
+
+    /**
+     * Can this model be exported?
+     *
+     * @return bool
+     */
+    public function can_export_configuration() : bool {
+
+        if (empty($this->model->timesplitting)) {
+            return false;
+        }
+        if (!$this->get_indicators()) {
+            return false;
+        }
+
+        if ($this->is_static()) {
+            return false;
+        }
+
+        return true;
+    }
+
     /**
      * Returns the model logs data.
      *
@@ -1370,6 +1539,29 @@ class model {
         return \core_analytics\dataset_manager::export_training_data($this->get_id(), $timesplittingid);
     }
 
+    /**
+     * Has the model been trained using data from this site?
+     *
+     * This method is useful to determine if a trained model can be evaluated as
+     * we can not use the same data for training and for evaluation.
+     *
+     * @return bool
+     */
+    public function trained_locally() : bool {
+        global $DB;
+
+        if (!$this->is_trained() || $this->is_static()) {
+            // Early exit.
+            return false;
+        }
+
+        if ($DB->record_exists('analytics_train_samples', ['modelid' => $this->model->id])) {
+            return true;
+        }
+
+        return false;
+    }
+
     /**
      * Flag the provided file as used for training or prediction.
      *
@@ -1395,14 +1587,16 @@ class model {
      * @param float $score
      * @param string $dir
      * @param array $info
+     * @param string $evaluationmode
      * @return int The inserted log id
      */
-    protected function log_result($timesplittingid, $score, $dir = false, $info = false) {
+    protected function log_result($timesplittingid, $score, $dir = false, $info = false, $evaluationmode = 'configuration') {
         global $DB, $USER;
 
         $log = new \stdClass();
         $log->modelid = $this->get_id();
         $log->version = $this->model->version;
+        $log->evaluationmode = $evaluationmode;
         $log->target = $this->model->target;
         $log->indicators = $this->model->indicators;
         $log->timesplitting = $timesplittingid;
@@ -1457,16 +1651,18 @@ class model {
         \core_analytics\manager::check_can_manage_models();
 
         // Delete current model version stored stuff.
-        $predictor = \core_analytics\manager::get_predictions_processor();
-        $predictor->clear_model($this->get_unique_id(), $this->get_output_dir());
-
-        $predictionids = $DB->get_fieldset_select('analytics_predictions', 'id', 'modelid = :modelid',
-            array('modelid' => $this->get_id()));
-        if ($predictionids) {
-            list($sql, $params) = $DB->get_in_or_equal($predictionids);
-            $DB->delete_records_select('analytics_prediction_actions', "predictionid $sql", $params);
+        $predictor = $this->get_predictions_processor(false);
+        if ($predictor->is_ready() !== true) {
+            $predictorname = \core_analytics\manager::get_predictions_processor_name($predictor);
+            debugging('Prediction processor ' . $predictorname . ' is not ready to be used. Model ' .
+                $this->model->id . ' could not be cleared.');
+        } else {
+            $predictor->clear_model($this->get_unique_id(), $this->get_output_dir());
         }
 
+        $DB->delete_records_select('analytics_prediction_actions', "predictionid IN
+            (SELECT id FROM {analytics_predictions} WHERE modelid = :modelid)", ['modelid' => $this->get_id()]);
+
         $DB->delete_records('analytics_predictions', array('modelid' => $this->model->id));
         $DB->delete_records('analytics_predict_samples', array('modelid' => $this->model->id));
         $DB->delete_records('analytics_train_samples', array('modelid' => $this->model->id));
@@ -1489,6 +1685,156 @@ class model {
         $DB->update_record('analytics_models', $this->model);
     }
 
+    /**
+     * Returns the name of the model.
+     *
+     * By default, models use their target's name as their own name. They can have their explicit name, too. In which
+     * case, the explicit name is used instead of the default one.
+     *
+     * @return string|lang_string
+     */
+    public function get_name() {
+
+        if (trim($this->model->name) === '') {
+            return $this->get_target()->get_name();
+
+        } else {
+            return $this->model->name;
+        }
+    }
+
+    /**
+     * Renames the model to the given name.
+     *
+     * When given an empty string, the model falls back to using the associated target's name as its name.
+     *
+     * @param string $name The new name for the model, empty string for using the default name.
+     */
+    public function rename(string $name) {
+        global $DB, $USER;
+
+        $this->model->name = $name;
+        $this->model->timemodified = time();
+        $this->model->usermodified = $USER->id;
+
+        $DB->update_record('analytics_models', $this->model);
+    }
+
+    /**
+     * Returns an inplace editable element with the model's name.
+     *
+     * @return \core\output\inplace_editable
+     */
+    public function inplace_editable_name() {
+
+        $displayname = format_string($this->get_name());
+
+        return new \core\output\inplace_editable('core_analytics', 'modelname', $this->model->id,
+            has_capability('moodle/analytics:managemodels', \context_system::instance()), $displayname, $this->model->name);
+    }
+
+    /**
+     * Adds the id from {analytics_predictions} db table to the prediction \stdClass objects.
+     *
+     * @param  \stdClass[] $predictionrecords
+     * @return \stdClass[] The prediction records including their ids in {analytics_predictions} db table.
+     */
+    private function add_prediction_ids($predictionrecords) {
+        global $DB;
+
+        $firstprediction = reset($predictionrecords);
+
+        $contextids = array_map(function($predictionobj) {
+            return $predictionobj->contextid;
+        }, $predictionrecords);
+
+        // Limited to 30000 records as a middle point between the ~65000 params limit in pgsql and the size limit for mysql which
+        // can be increased if required up to a reasonable point.
+        $chunks = array_chunk($contextids, 30000);
+        foreach ($chunks as $contextidschunk) {
+            list($contextsql, $contextparams) = $DB->get_in_or_equal($contextidschunk, SQL_PARAMS_NAMED);
+
+            // We select the fields that will allow us to map ids to $predictionrecords. Given that we already filter by modelid
+            // we have enough with sampleid and rangeindex. The reason is that the sampleid relation to a site is N - 1.
+            $fields = 'id, sampleid, rangeindex';
+
+            // We include the contextid and the timecreated filter to reduce the number of records in $dbpredictions. We can not
+            // add as many OR conditions as records in $predictionrecords.
+            $sql = "SELECT $fields
+                      FROM {analytics_predictions}
+                     WHERE modelid = :modelid
+                           AND contextid $contextsql
+                           AND timecreated >= :firsttimecreated";
+            $params = $contextparams + ['modelid' => $this->model->id, 'firsttimecreated' => $firstprediction->timecreated];
+            $dbpredictions = $DB->get_recordset_sql($sql, $params);
+            foreach ($dbpredictions as $id => $dbprediction) {
+                // The append_rangeindex implementation is the same regardless of the time splitting method in use.
+                $uniqueid = $this->get_time_splitting()->append_rangeindex($dbprediction->sampleid, $dbprediction->rangeindex);
+                $predictionrecords[$uniqueid]->id = $dbprediction->id;
+            }
+        }
+
+        return $predictionrecords;
+    }
+
+    /**
+     * Wrapper around analyser's get_samples to skip DB's max-number-of-params exception.
+     *
+     * @param  array  $sampleids
+     * @return array
+     */
+    public function get_samples(array $sampleids): array {
+
+        if (empty($sampleids)) {
+            throw new \coding_exception('No sample ids provided');
+        }
+
+        $chunksize = count($sampleids);
+
+        // We start with just 1 chunk, if it is too large for the db we split the list of sampleids in 2 and we
+        // try again. We repeat this process until the chunk is small enough for the db engine to process. The
+        // >= has been added in case there are other \dml_read_exceptions unrelated to the max number of params.
+        while (empty($done) && $chunksize >= 1) {
+
+            $chunks = array_chunk($sampleids, $chunksize);
+            $allsampleids = [];
+            $allsamplesdata = [];
+
+            foreach ($chunks as $index => $chunk) {
+
+                try {
+                    list($chunksampleids, $chunksamplesdata) = $this->get_analyser()->get_samples($chunk);
+                } catch (\dml_read_exception $e) {
+
+                    // Reduce the chunksize, we use floor() so the $chunksize is always less than the previous $chunksize value.
+                    $chunksize = floor($chunksize / 2);
+                    break;
+                }
+
+                // We can sum as these two arrays are indexed by sampleid and there are no collisions.
+                $allsampleids = $allsampleids + $chunksampleids;
+                $allsamplesdata = $allsamplesdata + $chunksamplesdata;
+
+                if ($index === count($chunks) - 1) {
+                    // We successfully processed all the samples in all chunks, we are done.
+                    $done = true;
+                }
+            }
+        }
+
+        if (empty($done)) {
+            if (!empty($e)) {
+                // Throw the last exception we caught, the \dml_read_exception we have been catching is unrelated to the max number
+                // of param's exception.
+                throw new \dml_read_exception($e);
+            } else {
+                throw new \coding_exception('We should never reach this point, there is a bug in ' .
+                    'core_analytics\\model::get_samples\'s code');
+            }
+        }
+        return [$allsampleids, $allsamplesdata];
+    }
+
     /**
      * Purges the insights cache.
      */
