@@ -86,10 +86,6 @@ defined('MOODLE_INTERNAL') || die();
 /**
  * This function checks all the requirements defined in environment.xml.
  *
- * @staticvar bool $result
- * @staticvar array $env_results
- * @staticvar bool $cache_exists
- *
  * @param string $version version to check.
  * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. Default ENV_SELECT_NEWER (BC)
  * @return array with two elements. The first element true/false, depending on
@@ -97,50 +93,36 @@ defined('MOODLE_INTERNAL') || die();
  *      objects that has detailed information about the checks and which ones passed.
  */
 function check_moodle_environment($version, $env_select = ENV_SELECT_NEWER) {
+    if ($env_select != ENV_SELECT_NEWER and $env_select != ENV_SELECT_DATAROOT and $env_select != ENV_SELECT_RELEASE) {
+        throw new coding_exception('Incorrect value of $env_select parameter');
+    }
 
-    $status = true;
+/// Get the more recent version before the requested
+    if (!$version = get_latest_version_available($version, $env_select)) {
+        return array(false, array());
+    }
 
-/// This are cached per request
-    static $result = true;
-    static $env_results;
-    static $cache_exists = false;
+/// Perform all the checks
+    if (!$environment_results = environment_check($version, $env_select)) {
+        return array(false, array());
+    }
 
-/// if we have results cached, use them
-    if ($cache_exists) {
-        $environment_results = $env_results;
-/// No cache exists, calculate everything
-    } else {
-    /// Get the more recent version before the requested
-        if (!$version = get_latest_version_available($version, $env_select)) {
-            $status = false;
+/// Iterate over all the results looking for some error in required items
+/// or some error_code
+    $result = true;
+    foreach ($environment_results as $environment_result) {
+        if (!$environment_result->getStatus() && $environment_result->getLevel() == 'required'
+          && !$environment_result->getBypassStr()) {
+            $result = false; // required item that is not bypased
+        } else if ($environment_result->getStatus() && $environment_result->getLevel() == 'required'
+          && $environment_result->getRestrictStr()) {
+            $result = false; // required item that is restricted
+        } else if ($environment_result->getErrorCode()) {
+            $result = false;
         }
+    }
 
-    /// Perform all the checks
-        if (!($environment_results = environment_check($version, $env_select)) && $status) {
-            $status = false;
-        }
-
-    /// Iterate over all the results looking for some error in required items
-    /// or some error_code
-        if ($status) {
-            foreach ($environment_results as $environment_result) {
-                if (!$environment_result->getStatus() && $environment_result->getLevel() == 'required'
-                  && !$environment_result->getBypassStr()) {
-                    $result = false; // required item that is not bypased
-                } else if ($environment_result->getStatus() && $environment_result->getLevel() == 'required'
-                  && $environment_result->getRestrictStr()) {
-                    $result = false; // required item that is restricted
-                } else if ($environment_result->getErrorCode()) {
-                    $result = false;
-                }
-            }
-        }
-    /// Going to end, we store environment_results to cache
-        $env_results = $environment_results;
-        $cache_exists = true;
-    } ///End of cache block
-
-    return array($result && $status, $environment_results);
+    return array($result, $environment_results);
 }
 
 
@@ -158,6 +140,7 @@ function environment_get_errors($environment_results) {
         $type = $environment_result->getPart();
         $info = $environment_result->getInfo();
         $status = $environment_result->getStatus();
+        $plugin = $environment_result->getPluginName();
         $error_code = $environment_result->getErrorCode();
 
         $a = new stdClass();
@@ -227,7 +210,13 @@ function environment_get_errors($environment_results) {
         // Append the restrict if there is some
         $feedbacktext .= $environment_result->strToReport($environment_result->getRestrictStr(), 'error');
 
-        $report .= html_to_text($feedbacktext);
+        if ($plugin === '') {
+            $report = '[' . get_string('coresystem') . '] ' . $report;
+        } else {
+            $report = '[' . $plugin . '] ' . $report;
+        }
+
+        $report .= ' - ' . html_to_text($feedbacktext);
 
         if ($environment_result->getPart() == 'custom_check'){
             $errors[] = array($info, $report);
@@ -270,68 +259,74 @@ function normalize_version($version) {
 /**
  * This function will load the environment.xml file and xmlize it
  *
- * @global object
- * @staticvar mixed $data
+ * @staticvar array $data
  * @uses ENV_SELECT_NEWER
  * @uses ENV_SELECT_DATAROOT
  * @uses ENV_SELECT_RELEASE
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. Default ENV_SELECT_NEWER (BC)
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return mixed the xmlized structure or false on error
  */
 function load_environment_xml($env_select=ENV_SELECT_NEWER) {
 
     global $CFG;
 
-    static $data; //Only load and xmlize once by request
+    static $data = array(); // Only load and xmlize once by request.
 
-    if (!empty($data)) {
-        return $data;
+    if (isset($data[$env_select])) {
+        return $data[$env_select];
     }
+    $contents = false;
 
-/// First of all, take a look inside $CFG->dataroot/environment/environment.xml
-    $file = $CFG->dataroot.'/environment/environment.xml';
-    $internalfile = $CFG->dirroot.'/'.$CFG->admin.'/environment.xml';
-    switch ($env_select) {
-        case ENV_SELECT_NEWER:
-            if (!is_file($file) || !is_readable($file) || filemtime($file) < filemtime($internalfile) ||
-                !$contents = file_get_contents($file)) {
-            /// Fallback to fixed $CFG->admin/environment.xml
-                if (!is_file($internalfile) || !is_readable($internalfile) || !$contents = file_get_contents($internalfile)) {
-                    return false;
+    if (is_numeric($env_select)) {
+        $file = $CFG->dataroot.'/environment/environment.xml';
+        $internalfile = $CFG->dirroot.'/'.$CFG->admin.'/environment.xml';
+        switch ($env_select) {
+            case ENV_SELECT_NEWER:
+                if (!is_file($file) || !is_readable($file) || filemtime($file) < filemtime($internalfile) ||
+                    !$contents = file_get_contents($file)) {
+                    /// Fallback to fixed $CFG->admin/environment.xml
+                    if (!is_file($internalfile) || !is_readable($internalfile) || !$contents = file_get_contents($internalfile)) {
+                        $contents = false;
+                    }
                 }
+                break;
+            case ENV_SELECT_DATAROOT:
+                if (!is_file($file) || !is_readable($file) || !$contents = file_get_contents($file)) {
+                    $contents = false;
+                }
+                break;
+            case ENV_SELECT_RELEASE:
+                if (!is_file($internalfile) || !is_readable($internalfile) || !$contents = file_get_contents($internalfile)) {
+                    $contents = false;
+                }
+                break;
+        }
+    } else {
+        if ($plugindir = core_component::get_component_directory($env_select)) {
+            $pluginfile = "$plugindir/environment.xml";
+            if (!is_file($pluginfile) || !is_readable($pluginfile) || !$contents = file_get_contents($pluginfile)) {
+                $contents = false;
             }
-            break;
-        case ENV_SELECT_DATAROOT:
-            if (!is_file($file) || !is_readable($file) || !$contents = file_get_contents($file)) {
-                return false;
-            }
-            break;
-        case ENV_SELECT_RELEASE:
-            if (!is_file($internalfile) || !is_readable($internalfile) || !$contents = file_get_contents($internalfile)) {
-                return false;
-            }
-            break;
+        }
+    }
+    // XML the whole file.
+    if ($contents !== false) {
+        $contents = xmlize($contents);
     }
-/// XML the whole file
-    $data = xmlize($contents);
 
-    return $data;
+    $data[$env_select] = $contents;
+
+    return $data[$env_select];
 }
 
 
 /**
  * This function will return the list of Moodle versions available
  *
- * @staticvar array $versions
- * @return mixed array of versions. False on error.
+ * @return array of versions
  */
-function get_list_of_environment_versions ($contents) {
-
-    static $versions = array();
-
-    if (!empty($versions)) {
-        return $versions;
-    }
+function get_list_of_environment_versions($contents) {
+    $versions = array();
 
     if (isset($contents['COMPATIBILITY_MATRIX']['#']['MOODLE'])) {
         foreach ($contents['COMPATIBILITY_MATRIX']['#']['MOODLE'] as $version) {
@@ -339,6 +334,10 @@ function get_list_of_environment_versions ($contents) {
         }
     }
 
+    if (isset($contents['COMPATIBILITY_MATRIX']['#']['PLUGIN'])) {
+        $versions[] = 'all';
+    }
+
     return $versions;
 }
 
@@ -351,7 +350,10 @@ function get_list_of_environment_versions ($contents) {
  * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
  * @return string|bool string more recent version or false if not found
  */
-function get_latest_version_available ($version, $env_select) {
+function get_latest_version_available($version, $env_select) {
+    if ($env_select != ENV_SELECT_NEWER and $env_select != ENV_SELECT_DATAROOT and $env_select != ENV_SELECT_RELEASE) {
+        throw new coding_exception('Incorrect value of $env_select parameter');
+    }
 
 /// Normalize the version requested
     $version = normalize_version($version);
@@ -387,7 +389,7 @@ function get_latest_version_available ($version, $env_select) {
  * This function will return the xmlized data belonging to one Moodle version
  *
  * @param string $version top version from which we start to look backwards
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return mixed the xmlized structure or false on error
  */
 function get_environment_for_version($version, $env_select) {
@@ -405,6 +407,14 @@ function get_environment_for_version($version, $env_select) {
         return false;
     }
 
+    // If $env_select is not numeric then this is being called on a plugin, and not the core environment.xml
+    // If a version of 'all' is in the arry is also means that the new <PLUGIN> tag was found, this should
+    // be matched against any version of Moodle.
+    if (!is_numeric($env_select) && in_array('all', $versions)
+            && environment_verify_plugin($env_select, $contents['COMPATIBILITY_MATRIX']['#']['PLUGIN'][0])) {
+        return $contents['COMPATIBILITY_MATRIX']['#']['PLUGIN'][0];
+    }
+
 /// If the version requested is available
     if (!in_array($version, $versions)) {
         return false;
@@ -416,6 +426,19 @@ function get_environment_for_version($version, $env_select) {
     return $contents['COMPATIBILITY_MATRIX']['#']['MOODLE'][$fl_arr[$version]];
 }
 
+/**
+ * Checks if a plugin tag has a name attribute and it matches the plugin being tested.
+ *
+ * @param string $plugin the name of the plugin.
+ * @param array $pluginxml the xmlised structure for the plugin tag being tested.
+ * @return boolean true if the name attribute exists and matches the plugin being tested.
+ */
+function environment_verify_plugin($plugin, $pluginxml) {
+    if (!isset($pluginxml['@']['name']) || $pluginxml['@']['name'] != $plugin) {
+        return false;
+    }
+    return true;
+}
 
 /**
  * This function will check for everything (DB, PHP and PHP extensions for now)
@@ -424,11 +447,15 @@ function get_environment_for_version($version, $env_select) {
  * @global object
  * @param string $version xml version we are going to use to test this server
  * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
- * @return array array of results encapsulated in one environment_result object
+ * @return environment_results[] array of results encapsulated in one environment_result object
  */
 function environment_check($version, $env_select) {
     global $CFG;
 
+    if ($env_select != ENV_SELECT_NEWER and $env_select != ENV_SELECT_DATAROOT and $env_select != ENV_SELECT_RELEASE) {
+        throw new coding_exception('Incorrect value of $env_select parameter');
+    }
+
 /// Normalize the version requested
     $version = normalize_version($version);
 
@@ -455,6 +482,58 @@ function environment_check($version, $env_select) {
     $custom_results = environment_custom_checks($version, $env_select);
     $results = array_merge($results, $custom_results);
 
+    // Always use the plugin directory version of environment.xml,
+    // add-on developers need to keep those up-to-date with future info.
+    foreach (core_component::get_plugin_types() as $plugintype => $unused) {
+        foreach (core_component::get_plugin_list_with_file($plugintype, 'environment.xml') as $pluginname => $unused) {
+            $plugin = $plugintype . '_' . $pluginname;
+
+            $result = environment_check_database($version, $plugin);
+            if ($result->error_code != NO_VERSION_DATA_FOUND
+                and $result->error_code != NO_DATABASE_SECTION_FOUND
+                and $result->error_code != NO_DATABASE_VENDORS_FOUND) {
+
+                $result->plugin = $plugin;
+                $results[] = $result;
+            }
+
+            $result = environment_check_php($version, $plugin);
+            if ($result->error_code != NO_VERSION_DATA_FOUND
+                and $result->error_code != NO_PHP_SECTION_FOUND
+                and $result->error_code != NO_PHP_VERSION_FOUND) {
+
+                $result->plugin = $plugin;
+                $results[] = $result;
+            }
+
+            $pluginresults = environment_check_php_extensions($version, $plugin);
+            foreach ($pluginresults as $result) {
+                if ($result->error_code != NO_VERSION_DATA_FOUND
+                    and $result->error_code != NO_PHP_EXTENSIONS_SECTION_FOUND) {
+
+                    $result->plugin = $plugin;
+                    $results[] = $result;
+                }
+            }
+
+            $pluginresults = environment_check_php_settings($version, $plugin);
+            foreach ($pluginresults as $result) {
+                if ($result->error_code != NO_VERSION_DATA_FOUND) {
+                    $result->plugin = $plugin;
+                    $results[] = $result;
+                }
+            }
+
+            $pluginresults = environment_custom_checks($version, $plugin);
+            foreach ($pluginresults as $result) {
+                if ($result->error_code != NO_VERSION_DATA_FOUND) {
+                    $result->plugin = $plugin;
+                    $results[] = $result;
+                }
+            }
+        }
+    }
+
     return $results;
 }
 
@@ -466,7 +545,7 @@ function environment_check($version, $env_select) {
  * @uses NO_PHP_EXTENSIONS_SECTION_FOUND
  * @uses NO_PHP_EXTENSIONS_NAME_FOUND
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return array array of results encapsulated in one environment_result object
  */
 function environment_check_php_extensions($version, $env_select) {
@@ -528,7 +607,7 @@ function environment_check_php_extensions($version, $env_select) {
  * @uses NO_VERSION_DATA_FOUND
  * @uses NO_PHP_SETTINGS_NAME_FOUND
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return array array of results encapsulated in one environment_result object
  */
 function environment_check_php_settings($version, $env_select) {
@@ -605,12 +684,11 @@ function environment_check_php_settings($version, $env_select) {
 /**
  * This function will do the custom checks.
  *
- * @global object
  * @uses CUSTOM_CHECK_FUNCTION_MISSING
  * @uses CUSTOM_CHECK_FILE_MISSING
  * @uses NO_CUSTOM_CHECK_FOUND
  * @param string $version xml version we are going to use to test this server.
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return array array of results encapsulated in environment_result objects.
  */
 function environment_custom_checks($version, $env_select) {
@@ -642,27 +720,31 @@ function environment_custom_checks($version, $env_select) {
         $level = get_level($check);
 
     /// Check for extension name
-        if (isset($check['@']['file']) && isset($check['@']['function'])) {
-            $file = $CFG->dirroot . '/' . $check['@']['file'];
+        if (isset($check['@']['function'])) {
             $function = $check['@']['function'];
-            if (is_readable($file)) {
-                include_once($file);
-                if (function_exists($function)) {
-                    $result->setLevel($level);
+            $file = null;
+            if (isset($check['@']['file'])) {
+                $file = $CFG->dirroot . '/' . $check['@']['file'];
+                if (is_readable($file)) {
+                    include_once($file);
+                }
+            }
+
+            if (is_callable($function)) {
+                $result->setLevel($level);
+                $result->setInfo($function);
+                $result = call_user_func($function, $result);
+            } else if (!$file or is_readable($file)) {
+            /// Only show error for current version (where function MUST exist)
+            /// else, we are performing custom checks against future versiosn
+            /// and function MAY not exist, so it doesn't cause error, just skip
+            /// custom check by returning null. MDL-15939
+                if (version_compare($current_version, $version, '>=')) {
+                    $result->setStatus(false);
                     $result->setInfo($function);
-                    $result = $function($result);
+                    $result->setErrorCode(CUSTOM_CHECK_FUNCTION_MISSING);
                 } else {
-                /// Only show error for current version (where function MUST exist)
-                /// else, we are performing custom checks against future versiosn
-                /// and function MAY not exist, so it doesn't cause error, just skip
-                /// custom check by returning null. MDL-15939
-                    if (version_compare($current_version, $version, '>=')) {
-                        $result->setStatus(false);
-                        $result->setInfo($function);
-                        $result->setErrorCode(CUSTOM_CHECK_FUNCTION_MISSING);
-                    } else {
-                        $result = null;
-                    }
+                    $result = null;
                 }
             } else {
             /// Only show error for current version (where function MUST exist)
@@ -699,7 +781,7 @@ function environment_custom_checks($version, $env_select) {
  *
  * @uses NO_VERSION_DATA_FOUND
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return object results encapsulated in one environment_result object
  */
 function environment_check_moodle($version, $env_select) {
@@ -750,7 +832,7 @@ function environment_check_moodle($version, $env_select) {
  * @uses NO_PHP_SECTION_FOUND
  * @uses NO_PHP_VERSION_FOUND
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return object results encapsulated in one environment_result object
  */
 function environment_check_php($version, $env_select) {
@@ -805,7 +887,7 @@ function environment_check_php($version, $env_select) {
 /**
  * Looks for buggy PCRE implementation, we need unicode support in Moodle...
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return stdClass results encapsulated in one environment_result object, null if irrelevant
  */
 function environment_check_pcre_unicode($version, $env_select) {
@@ -846,11 +928,10 @@ function environment_check_pcre_unicode($version, $env_select) {
 /**
  * This function will check if unicode database requirements are satisfied
  *
- * @global object
  * @uses NO_VERSION_DATA_FOUND
  * @uses NO_UNICODE_SECTION_FOUND
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return object results encapsulated in one environment_result object
  */
 function environment_check_unicode($version, $env_select) {
@@ -895,7 +976,6 @@ function environment_check_unicode($version, $env_select) {
 /**
  * This function will check if database requirements are satisfied
  *
- * @global object
  * @uses NO_VERSION_DATA_FOUND
  * @uses NO_DATABASE_SECTION_FOUND
  * @uses NO_DATABASE_VENDORS_FOUND
@@ -903,7 +983,7 @@ function environment_check_unicode($version, $env_select) {
  * @uses NO_DATABASE_VENDOR_POSTGRES_FOUND
  * @uses NO_DATABASE_VENDOR_VERSION_FOUND
  * @param string $version xml version we are going to use to test this server
- * @param int $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use.
+ * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
  * @return object results encapsulated in one environment_result object
  */
 function environment_check_database($version, $env_select) {
@@ -975,6 +1055,19 @@ function environment_check_database($version, $env_select) {
         return $result;
     }
 
+    // Check if the DB Vendor has been properly configured.
+    // Hack: this is required when playing with MySQL and MariaDB since they share the same PHP module and base DB classes,
+    // whilst they are slowly evolving using separate directions though MariaDB is still an "almost" drop-in replacement.
+    $dbvendorismysql = ($current_vendor === 'mysql');
+    $dbtypeismariadb = (stripos($dbinfo['description'], 'mariadb') !== false);
+    if ($dbvendorismysql && $dbtypeismariadb) {
+        $result->setStatus(false);
+        $result->setLevel($level);
+        $result->setInfo($current_vendor . ' (' . $dbinfo['description'] . ')');
+        $result->setFeedbackStr('environmentmariadbwrongdbtype');
+        return $result;
+    }
+
 /// And finally compare them, saving results
     if (version_compare($current_version, $needed_version, '>=')) {
         $result->setStatus(true);
@@ -984,7 +1077,7 @@ function environment_check_database($version, $env_select) {
     $result->setLevel($level);
     $result->setCurrentVersion($current_version);
     $result->setNeededVersion($needed_version);
-    $result->setInfo($current_vendor);
+    $result->setInfo($current_vendor . ' (' . $dbinfo['description'] . ')');
 
 /// Do any actions defined in the XML file.
     process_environment_result($vendorsxml[$current_vendor], $result);
@@ -1023,7 +1116,11 @@ function process_environment_bypass($xml, &$result) {
             if ($function($result)) {
             /// We only set the bypass message if the function itself hasn't defined it before
                 if (empty($result->getBypassStr)) {
-                    $result->setBypassStr($message);
+                    if (isset($xml['#']['BYPASS'][0]['@']['plugin'])) {
+                        $result->setBypassStr(array($message, $xml['#']['BYPASS'][0]['@']['plugin']));
+                    } else {
+                        $result->setBypassStr($message);
+                    }
                 }
             }
         }
@@ -1059,7 +1156,11 @@ function process_environment_restrict($xml, &$result) {
             if ($function($result)) {
             /// We only set the restrict message if the function itself hasn't defined it before
                 if (empty($result->getRestrictStr)) {
-                    $result->setRestrictStr($message);
+                    if (isset($xml['#']['RESTRICT'][0]['@']['plugin'])) {
+                        $result->setRestrictStr(array($message, $xml['#']['RESTRICT'][0]['@']['plugin']));
+                    } else {
+                        $result->setRestrictStr($message);
+                    }
                 }
             }
         }
@@ -1092,15 +1193,27 @@ function process_environment_messages($xml, &$result) {
 
         if (!$result->status and $result->getLevel() == 'required') {
             if (isset($feedbackxml['ON_ERROR'][0]['@']['message'])) {
-                $result->setFeedbackStr($feedbackxml['ON_ERROR'][0]['@']['message']);
+                if (isset($feedbackxml['ON_ERROR'][0]['@']['plugin'])) {
+                    $result->setFeedbackStr(array($feedbackxml['ON_ERROR'][0]['@']['message'], $feedbackxml['ON_ERROR'][0]['@']['plugin']));
+                } else {
+                    $result->setFeedbackStr($feedbackxml['ON_ERROR'][0]['@']['message']);
+                }
             }
         } else if (!$result->status and $result->getLevel() == 'optional') {
             if (isset($feedbackxml['ON_CHECK'][0]['@']['message'])) {
-                $result->setFeedbackStr($feedbackxml['ON_CHECK'][0]['@']['message']);
+                if (isset($feedbackxml['ON_CHECK'][0]['@']['plugin'])) {
+                    $result->setFeedbackStr(array($feedbackxml['ON_CHECK'][0]['@']['message'], $feedbackxml['ON_CHECK'][0]['@']['plugin']));
+                } else {
+                    $result->setFeedbackStr($feedbackxml['ON_CHECK'][0]['@']['message']);
+                }
             }
         } else {
             if (isset($feedbackxml['ON_OK'][0]['@']['message'])) {
-                $result->setFeedbackStr($feedbackxml['ON_OK'][0]['@']['message']);
+                if (isset($feedbackxml['ON_OK'][0]['@']['plugin'])) {
+                    $result->setFeedbackStr(array($feedbackxml['ON_OK'][0]['@']['message'], $feedbackxml['ON_OK'][0]['@']['plugin']));
+                } else {
+                    $result->setFeedbackStr($feedbackxml['ON_OK'][0]['@']['message']);
+                }
             }
         }
     }
@@ -1123,7 +1236,7 @@ class environment_results {
      */
     var $part;
     /**
-     * @var bool
+     * @var bool true means the test passed and all is OK. false means it failed.
      */
     var $status;
     /**
@@ -1158,13 +1271,16 @@ class environment_results {
      * @var string String to show if some restrict has happened
      */
     var $restrict_str;
-
+    /**
+     * @var string|null full plugin name or null if main environment
+     */
+    var $plugin = null;
     /**
      * Constructor of the environment_result class. Just set default values
      *
      * @param string $part
      */
-    function environment_results($part) {
+    public function __construct($part) {
         $this->part=$part;
         $this->status=false;
         $this->error_code=NO_ERROR;
@@ -1177,14 +1293,24 @@ class environment_results {
         $this->restrict_str='';
     }
 
+    /**
+     * Old syntax of class constructor. Deprecated in PHP7.
+     *
+     * @deprecated since Moodle 3.1
+     */
+    public function environment_results($part) {
+        debugging('Use of class name as constructor is deprecated', DEBUG_DEVELOPER);
+        self::__construct($part);
+    }
+
     /**
      * Set the status
      *
-     * @param boolean $status the status (true/false)
+     * @param bool $testpassed true means the test passed and all is OK. false means it failed.
      */
-    function setStatus($status) {
-        $this->status=$status;
-        if ($status) {
+    function setStatus($testpassed) {
+        $this->status = $testpassed;
+        if ($testpassed) {
             $this->setErrorCode(NO_ERROR);
         }
     }
@@ -1274,7 +1400,7 @@ class environment_results {
     /**
      * Get the status
      *
-     * @return boolean result
+     * @return bool true means the test passed and all is OK. false means it failed.
      */
     function getStatus() {
         return $this->status;
@@ -1385,47 +1511,27 @@ class environment_results {
             return '';
         }
     }
-}
 
-/// Here all the bypass functions are coded to be used by the environment
-/// checker. All those functions will receive the result object and will
-/// return it modified as needed (status and bypass string)
-
-/**
- * This function will bypass MySQL 4.1.16 reqs if:
- *   - We are using MySQL > 4.1.12, informing about problems with non latin chars in the future
- *
- * @param object result object to handle
- * @return boolean true/false to determinate if the bypass has to be performed (true) or no (false)
- */
-function bypass_mysql416_reqs ($result) {
-/// See if we are running MySQL >= 4.1.12
-    if (version_compare($result->getCurrentVersion(), '4.1.12', '>=')) {
-        return true;
+    /**
+     * Get plugin name.
+     *
+     * @return string plugin name
+     */
+    function getPluginName() {
+        if ($this->plugin) {
+            $manager = core_plugin_manager::instance();
+            list($plugintype, $pluginname) = core_component::normalize_component($this->plugin);
+            return $manager->plugintype_name($plugintype) . ' / ' . $manager->plugin_name($this->plugin);
+        } else {
+            return '';
+        }
     }
-
-    return false;
 }
 
 /// Here all the restrict functions are coded to be used by the environment
 /// checker. All those functions will receive the result object and will
 /// return it modified as needed (status and bypass string)
 
-/**
- * This function will restrict PHP reqs if:
- *   - We are using PHP 5.0.x, informing about the buggy version
- *
- * @param object $result object to handle
- * @return boolean true/false to determinate if the restrict has to be performed (true) or no (false)
- */
-function restrict_php50_version($result) {
-    if (version_compare($result->getCurrentVersion(), '5.0.0', '>=')
-      and version_compare($result->getCurrentVersion(), '5.0.99', '<')) {
-        return true;
-    }
-    return false;
-}
-
 /**
  * @param array $element the element from the environment.xml file that should have
  *      either a level="required" or level="optional" attribute.
@@ -1462,3 +1568,81 @@ function process_environment_result($element, &$result) {
 /// Process restrict, modifying $result if needed.
     process_environment_restrict($element, $result);
 }
+
+/**
+ * Check if the current PHP version is greater than or equal to
+ * PHP version 7.
+ *
+ * @param object $result an environment_results instance
+ * @return bool result of version check
+ */
+function restrict_php_version_7(&$result) {
+    return restrict_php_version($result, '7');
+}
+
+/**
+ * Check if the current PHP version is greater than or equal to an
+ * unsupported version.
+ *
+ * @param object $result an environment_results instance
+ * @param string $version the version of PHP that can't be used
+ * @return bool result of version check
+ */
+function restrict_php_version(&$result, $version) {
+
+    // Get the current PHP version.
+    $currentversion = normalize_version(phpversion());
+
+    // Confirm we're using a supported PHP version.
+    if (version_compare($currentversion, $version, '<')) {
+        // Everything is ok, the restriction doesn't apply.
+        return false;
+    } else {
+        // We're using an unsupported PHP version, apply restriction.
+        return true;
+    }
+}
+
+/**
+ * Check if the current PHP version is greater than or equal to
+ * PHP version 7.1.
+ *
+ * @param object $result an environment_results instance
+ * @return bool result of version check
+ */
+function restrict_php_version_71(&$result) {
+    return restrict_php_version($result, '7.1');
+}
+
+/**
+ * Check if the current PHP version is greater than or equal to
+ * PHP version 7.2.
+ *
+ * @param object $result an environment_results instance
+ * @return bool result of version check
+ */
+function restrict_php_version_72(&$result) {
+    return restrict_php_version($result, '7.2');
+}
+
+/**
+ * Check if the current PHP version is greater than or equal to
+ * PHP version 7.3.
+ *
+ * @param object $result an environment_results instance
+ * @return bool result of version check
+ */
+function restrict_php_version_73(&$result) {
+    return restrict_php_version($result, '7.3');
+}
+
+/**
+ * Check if the current PHP version is greater than or equal to
+ * PHP version 7.4.
+ *
+ * @param object $result an environment_results instance
+ * @return bool result of version check
+ */
+function restrict_php_version_74(&$result) {
+    return restrict_php_version($result, '7.4');
+}
