@@ -256,6 +256,27 @@ class core_externallib_testcase extends advanced_testcase {
         $settings->set_filter($currentfilter);
     }
 
+    /**
+     * Test for clean_returnvalue() for testing that returns the PHP type.
+     */
+    public function test_clean_returnvalue_return_php_type() {
+
+        $returndesc = new external_single_structure(
+            array(
+                'value' => new external_value(PARAM_RAW, 'Some text', VALUE_OPTIONAL, null, NULL_NOT_ALLOWED)
+            )
+        );
+
+        // Check return type on exception because the external values does not allow NULL values.
+        $testdata = array('value' => null);
+        try {
+            $cleanedvalue = external_api::clean_returnvalue($returndesc, $testdata);
+        } catch (moodle_exception $e) {
+            $this->assertInstanceOf('invalid_response_exception', $e);
+            $this->assertContains('of PHP type "NULL"', $e->debuginfo);
+        }
+    }
+
     /**
      * Test for clean_returnvalue().
      */
@@ -490,7 +511,7 @@ class core_externallib_testcase extends advanced_testcase {
 
 
     public function test_call_external_function() {
-        global $PAGE, $COURSE;
+        global $PAGE, $COURSE, $CFG;
 
         $this->resetAfterTest(true);
 
@@ -525,6 +546,16 @@ class core_externallib_testcase extends advanced_testcase {
 
         $this->assertSame($beforepage, $PAGE);
         $this->assertSame($beforecourse, $COURSE);
+
+        // Test a function that triggers a PHP exception.
+        require_once($CFG->dirroot . '/lib/tests/fixtures/test_external_function_throwable.php');
+
+        // Call our test function.
+        $result = test_external_function_throwable::call_external_function('core_throw_exception', array(), false);
+
+        $this->assertTrue($result['error']);
+        $this->assertArrayHasKey('exception', $result);
+        $this->assertEquals($result['exception']->message, 'Exception - Modulo by zero');
     }
 
     /**
@@ -570,6 +601,11 @@ class core_externallib_testcase extends advanced_testcase {
         $files = external_util::get_area_files($context, $component, $filearea, false);
         $this->assertEquals($expectedfiles, $files);
 
+        $DB->method('get_in_or_equal')->willReturn([
+            '= :mock1',
+            ['mock1' => $itemid]
+        ]);
+
         // Get just the file indicated by $itemid.
         $files = external_util::get_area_files($context, $component, $filearea, $itemid);
         $this->assertEquals($expectedfiles, $files);
