@@ -22,10 +22,9 @@
 //                                                                       //
 ///////////////////////////////////////////////////////////////////////////
 
-    require_once(dirname(__FILE__) . '/../../config.php');
-    require_once($CFG->dirroot . '/mod/data/lib.php');
+    require_once(__DIR__ . '/../../config.php');
+    require_once($CFG->dirroot . '/mod/data/locallib.php');
     require_once($CFG->libdir . '/rsslib.php');
-    require_once($CFG->libdir . '/completionlib.php');
 
 /// One of these is necessary!
     $id = optional_param('id', 0, PARAM_INT);  // course module id
@@ -133,8 +132,6 @@
 
     if (!empty($advanced)) {
         $search = '';
-        $vals = array();
-        $fields = $DB->get_records('data_fields', array('dataid'=>$data->id));
 
         //Added to ammend paging error. This error would occur when attempting to go from one page of advanced
         //search results to another.  All fields were reset in the page transfer, and there was no way of determining
@@ -156,71 +153,10 @@
         else {
             $paging = true;
         }
-        if (!empty($fields)) {
-            foreach($fields as $field) {
-                $searchfield = data_get_field_from_id($field->id, $data);
-                //Get field data to build search sql with.  If paging is false, get from user.
-                //If paging is true, get data from $search_array which is obtained from the $SESSION (see line 116).
-                if(!$paging) {
-                    $val = $searchfield->parse_search_field();
-                } else {
-                    //Set value from session if there is a value @ the required index.
-                    if (isset($search_array[$field->id])) {
-                        $val = $search_array[$field->id]->data;
-                    } else {             //If there is not an entry @ the required index, set value to blank.
-                        $val = '';
-                    }
-                }
-                if (!empty($val)) {
-                    $search_array[$field->id] = new stdClass();
-                    list($search_array[$field->id]->sql, $search_array[$field->id]->params) = $searchfield->generate_sql('c'.$field->id, $val);
-                    $search_array[$field->id]->data = $val;
-                    $vals[] = $val;
-                } else {
-                    // clear it out
-                    unset($search_array[$field->id]);
-                }
-            }
-        }
 
-        if (!$paging) {
-            // name searching
-            $fn = optional_param('u_fn', '', PARAM_NOTAGS);
-            $ln = optional_param('u_ln', '', PARAM_NOTAGS);
-        } else {
-            $fn = isset($search_array[DATA_FIRSTNAME]) ? $search_array[DATA_FIRSTNAME]->data : '';
-            $ln = isset($search_array[DATA_LASTNAME]) ? $search_array[DATA_LASTNAME]->data : '';
-        }
-        if (!empty($fn)) {
-            $search_array[DATA_FIRSTNAME] = new stdClass();
-            $search_array[DATA_FIRSTNAME]->sql    = '';
-            $search_array[DATA_FIRSTNAME]->params = array();
-            $search_array[DATA_FIRSTNAME]->field  = 'u.firstname';
-            $search_array[DATA_FIRSTNAME]->data   = $fn;
-            $vals[] = $fn;
-        } else {
-            unset($search_array[DATA_FIRSTNAME]);
-        }
-        if (!empty($ln)) {
-            $search_array[DATA_LASTNAME] = new stdClass();
-            $search_array[DATA_LASTNAME]->sql     = '';
-            $search_array[DATA_LASTNAME]->params = array();
-            $search_array[DATA_LASTNAME]->field   = 'u.lastname';
-            $search_array[DATA_LASTNAME]->data    = $ln;
-            $vals[] = $ln;
-        } else {
-            unset($search_array[DATA_LASTNAME]);
-        }
-
-        $SESSION->dataprefs[$data->id]['search_array'] = $search_array;     // Make it sticky
-
-        // in case we want to switch to simple search later - there might be multiple values there ;-)
-        if ($vals) {
-            $val = reset($vals);
-            if (is_string($val)) {
-                $search = $val;
-            }
-        }
+        // Now build the advanced search array.
+        list($search_array, $search) = data_build_search_array($data, $paging, $search_array);
+        $SESSION->dataprefs[$data->id]['search_array'] = $search_array;     // Make it sticky.
 
     } else {
         $search = optional_param('search', $SESSION->dataprefs[$data->id]['search'], PARAM_NOTAGS);
@@ -233,9 +169,6 @@
         $search = '';
     }
 
-    if (core_text::strlen($search) < 2) {
-        $search = '';
-    }
     $SESSION->dataprefs[$data->id]['search'] = $search;   // Make it sticky
 
     $sort = optional_param('sort', $SESSION->dataprefs[$data->id]['sort'], PARAM_INT);
@@ -255,8 +188,8 @@
         set_user_preference('data_perpage_'.$data->id, $perpage);
     }
 
-    add_to_log($course->id, 'data', 'view', "view.php?id=$cm->id", $data->id, $cm->id);
-
+    // Completion and trigger events.
+    data_view($data, $course, $cm, $context);
 
     $urlparams = array('d' => $data->id);
     if ($record) {
@@ -293,10 +226,6 @@
         $PAGE->requires->js('/mod/data/js.php?d='.$data->id, true);
     }
 
-    // Mark as viewed
-    $completion = new completion_info($course);
-    $completion->set_module_viewed($cm);
-
 /// Print the page header
     // Note: MDL-19010 there will be further changes to printing header and blocks.
     // The code will be much nicer than this eventually.
@@ -319,28 +248,22 @@
         $PAGE->navbar->add(get_string('search'));
     }
 
+    $PAGE->force_settings_menu();
     $PAGE->set_title($title);
     $PAGE->set_heading($course->fullname);
 
     echo $OUTPUT->header();
 
-/// Check to see if groups are being used here
-    $currentgroup = groups_get_activity_group($cm);
+    // Check to see if groups are being used here.
+    // We need the most up to date current group value. Make sure it is updated at this point.
+    $currentgroup = groups_get_activity_group($cm, true);
     $groupmode = groups_get_activity_groupmode($cm);
     $canmanageentries = has_capability('mod/data:manageentries', $context);
-    // If a student is not part of a group and seperate groups is enabled, we don't
-    // want them seeing all records.
-    if ($currentgroup == 0 && $groupmode == 1 && !$canmanageentries) {
-        $canviewallrecords = false;
-    } else {
-        $canviewallrecords = true;
-    }
 
-    // detect entries not approved yet and show hint instead of not found error
-    if ($record and $data->approval and !$record->approved and $record->userid != $USER->id and !$canmanageentries) {
-        if (!$currentgroup or $record->groupid == $currentgroup or $record->groupid == 0) {
-            print_error('notapproved', 'data');
-        }
+
+    // Detect entries not approved yet and show hint instead of not found error.
+    if ($record and !data_can_view_record($data, $record, $currentgroup, $canmanageentries)) {
+        print_error('notapproved', 'data');
     }
 
     echo $OUTPUT->heading(format_string($data->name), 2);
@@ -365,13 +288,15 @@
 
 /// Delete any requested records
 
-    if ($delete && confirm_sesskey() && ($canmanageentries or data_isowner($delete))) {
+    if ($delete && confirm_sesskey() && (data_user_can_manage_entry($delete, $data, $context))) {
         if ($confirm = optional_param('confirm',0,PARAM_INT)) {
             if (data_delete_record($delete, $data, $course->id, $cm->id)) {
                 echo $OUTPUT->notification(get_string('recorddeleted','data'), 'notifysuccess');
             }
         } else {   // Print a confirmation page
-            $allnamefields = get_all_user_name_fields(true, 'u');
+            $allnamefields = user_picture::fields('u');
+            // Remove the id from the string. This already exists in the sql statement.
+            $allnamefields = str_replace('u.id,', '', $allnamefields);
             $dbparams = array($delete);
             if ($deleterecord = $DB->get_record_sql("SELECT dr.*, $allnamefields
                                                        FROM {data_records} dr
@@ -407,14 +332,20 @@
             $validrecords = array();
             $recordids = array();
             foreach ($multidelete as $value) {
-                if ($deleterecord = $DB->get_record('data_records', array('id'=>$value))) {   // Need to check this is valid
-                    if ($deleterecord->dataid == $data->id) {                       // Must be from this database
+                $allnamefields = user_picture::fields('u');
+                // Remove the id from the string. This already exists in the sql statement.
+                $allnamefields = str_replace('u.id,', '', $allnamefields);
+                $dbparams = array('id' => $value);
+                if ($deleterecord = $DB->get_record_sql("SELECT dr.*, $allnamefields
+                                                           FROM {data_records} dr
+                                                           JOIN {user} u ON dr.userid = u.id
+                                                          WHERE dr.id = ?", $dbparams)) { // Need to check this is valid.
+                    if ($deleterecord->dataid == $data->id) {  // Must be from this database.
                         $validrecords[] = $deleterecord;
                         $recordids[] = $deleterecord->id;
                     }
                 }
             }
-
             $serialiseddata = json_encode($recordids);
             $submitactions = array('d' => $data->id, 'sesskey' => sesskey(), 'confirm' => '1', 'serialdelete' => $serialiseddata);
             $action = new moodle_url('/mod/data/view.php', $submitactions);
@@ -427,19 +358,13 @@
         }
     }
 
+    // If data activity closed dont let students in.
+    list($showactivity, $warnings) = data_get_time_availability_status($data, $canmanageentries);
 
-//if data activity closed dont let students in
-$showactivity = true;
-if (!$canmanageentries) {
-    $timenow = time();
-    if (!empty($data->timeavailablefrom) && $data->timeavailablefrom > $timenow) {
-        echo $OUTPUT->notification(get_string('notopenyet', 'data', userdate($data->timeavailablefrom)));
-        $showactivity = false;
-    } else if (!empty($data->timeavailableto) && $timenow > $data->timeavailableto) {
-        echo $OUTPUT->notification(get_string('expired', 'data', userdate($data->timeavailableto)));
-        $showactivity = false;
+    if (!$showactivity) {
+        $reason = current(array_keys($warnings));
+        echo $OUTPUT->notification(get_string($reason, 'data', $warnings[$reason]));
     }
-}
 
 if ($showactivity) {
     // Print the tabs
@@ -453,263 +378,48 @@ if ($showactivity) {
     }
     include('tabs.php');
 
-    $url = new moodle_url('/mod/data/view.php', array('d' => $data->id, 'sesskey' => sesskey()));
-    echo html_writer::start_tag('form', array('action' => $url, 'method' => 'post'));
-
     if ($mode == 'asearch') {
         $maxcount = 0;
+        data_print_preference_form($data, $perpage, $search, $sort, $order, $search_array, $advanced, $mode);
 
     } else {
         // Approve or disapprove any requested records
-        $params = array(); // named params array
-
         $approvecap = has_capability('mod/data:approve', $context);
 
         if (($approve || $disapprove) && confirm_sesskey() && $approvecap) {
-            $newapproved = $approve ? 1 : 0;
+            $newapproved = $approve ? true : false;
             $recordid = $newapproved ? $approve : $disapprove;
             if ($approverecord = $DB->get_record('data_records', array('id' => $recordid))) {   // Need to check this is valid
                 if ($approverecord->dataid == $data->id) {                       // Must be from this database
-                    $newrecord = new stdClass();
-                    $newrecord->id = $approverecord->id;
-                    $newrecord->approved = $newapproved;
-                    $DB->update_record('data_records', $newrecord);
+                    data_approve_entry($approverecord->id, $newapproved);
                     $msgkey = $newapproved ? 'recordapproved' : 'recorddisapproved';
                     echo $OUTPUT->notification(get_string($msgkey, 'data'), 'notifysuccess');
                 }
             }
         }
 
-         $numentries = data_numentries($data);
+        $numentries = data_numentries($data);
     /// Check the number of entries required against the number of entries already made (doesn't apply to teachers)
-        if ($data->requiredentries > 0 && $numentries < $data->requiredentries && !$canmanageentries) {
-            $data->entriesleft = $data->requiredentries - $numentries;
+        if ($data->entriesleft = data_get_entries_left_to_add($data, $numentries, $canmanageentries)) {
             $strentrieslefttoadd = get_string('entrieslefttoadd', 'data', $data);
             echo $OUTPUT->notification($strentrieslefttoadd);
         }
 
     /// Check the number of entries required before to view other participant's entries against the number of entries already made (doesn't apply to teachers)
         $requiredentries_allowed = true;
-        if ($data->requiredentriestoview > 0 && $numentries < $data->requiredentriestoview && !$canmanageentries) {
-            $data->entrieslefttoview = $data->requiredentriestoview - $numentries;
+        if ($data->entrieslefttoview = data_get_entries_left_to_view($data, $numentries, $canmanageentries)) {
             $strentrieslefttoaddtoview = get_string('entrieslefttoaddtoview', 'data', $data);
             echo $OUTPUT->notification($strentrieslefttoaddtoview);
             $requiredentries_allowed = false;
         }
 
-        // Initialise the first group of params for advanced searches.
-        $initialparams   = array();
+        // Search for entries.
+        list($records, $maxcount, $totalcount, $page, $nowperpage, $sort, $mode) =
+            data_search_entries($data, $cm, $context, $mode, $currentgroup, $search, $sort, $order, $page, $perpage, $advanced, $search_array, $record);
 
-    /// setup group and approve restrictions
-        if (!$approvecap && $data->approval) {
-            if (isloggedin()) {
-                $approveselect = ' AND (r.approved=1 OR r.userid=:myid1) ';
-                $params['myid1'] = $USER->id;
-                $initialparams['myid1'] = $params['myid1'];
-            } else {
-                $approveselect = ' AND r.approved=1 ';
-            }
-        } else {
-            $approveselect = ' ';
-        }
-
-        if ($currentgroup) {
-            $groupselect = " AND (r.groupid = :currentgroup OR r.groupid = 0)";
-            $params['currentgroup'] = $currentgroup;
-            $initialparams['currentgroup'] = $params['currentgroup'];
-        } else {
-            if ($canviewallrecords) {
-                $groupselect = ' ';
-            } else {
-                // If separate groups are enabled and the user isn't in a group or
-                // a teacher, manager, admin etc, then just show them entries for 'All participants'.
-                $groupselect = " AND r.groupid = 0";
-            }
-        }
-
-        // Init some variables to be used by advanced search
-        $advsearchselect = '';
-        $advwhere        = '';
-        $advtables       = '';
-        $advparams       = array();
-        // This is used for the initial reduction of advanced search results with required entries.
-        $entrysql        = '';
-        $namefields = get_all_user_name_fields(true, 'u');
-
-    /// Find the field we are sorting on
-        if ($sort <= 0 or !$sortfield = data_get_field_from_id($sort, $data)) {
-
-            switch ($sort) {
-                case DATA_LASTNAME:
-                    $ordering = "u.lastname $order, u.firstname $order";
-                    break;
-                case DATA_FIRSTNAME:
-                    $ordering = "u.firstname $order, u.lastname $order";
-                    break;
-                case DATA_APPROVED:
-                    $ordering = "r.approved $order, r.timecreated $order";
-                    break;
-                case DATA_TIMEMODIFIED:
-                    $ordering = "r.timemodified $order";
-                    break;
-                case DATA_TIMEADDED:
-                default:
-                    $sort     = 0;
-                    $ordering = "r.timecreated $order";
-            }
-
-            $what = ' DISTINCT r.id, r.approved, r.timecreated, r.timemodified, r.userid, ' . $namefields;
-            $count = ' COUNT(DISTINCT c.recordid) ';
-            $tables = '{data_content} c,{data_records} r, {user} u ';
-            $where =  'WHERE c.recordid = r.id
-                         AND r.dataid = :dataid
-                         AND r.userid = u.id ';
-            $params['dataid'] = $data->id;
-            $sortorder = ' ORDER BY '.$ordering.', r.id ASC ';
-            $searchselect = '';
-
-            // If requiredentries is not reached, only show current user's entries
-            if (!$requiredentries_allowed) {
-                $where .= ' AND u.id = :myid2 ';
-                $entrysql = ' AND r.userid = :myid3 ';
-                $params['myid2'] = $USER->id;
-                $initialparams['myid3'] = $params['myid2'];
-            }
-
-            if (!empty($advanced)) {                                                  //If advanced box is checked.
-                $i = 0;
-                foreach($search_array as $key => $val) {                              //what does $search_array hold?
-                    if ($key == DATA_FIRSTNAME or $key == DATA_LASTNAME) {
-                        $i++;
-                        $searchselect .= " AND ".$DB->sql_like($val->field, ":search_flname_$i", false);
-                        $params['search_flname_'.$i] = "%$val->data%";
-                        continue;
-                    }
-                    $advtables .= ', {data_content} c'.$key.' ';
-                    $advwhere .= ' AND c'.$key.'.recordid = r.id';
-                    $advsearchselect .= ' AND ('.$val->sql.') ';
-                    $advparams = array_merge($advparams, $val->params);
-                }
-            } else if ($search) {
-                $searchselect = " AND (".$DB->sql_like('c.content', ':search1', false)."
-                                  OR ".$DB->sql_like('u.firstname', ':search2', false)."
-                                  OR ".$DB->sql_like('u.lastname', ':search3', false)." ) ";
-                $params['search1'] = "%$search%";
-                $params['search2'] = "%$search%";
-                $params['search3'] = "%$search%";
-            } else {
-                $searchselect = ' ';
-            }
-
-        } else {
-
-            $sortcontent = $DB->sql_compare_text('c.' . $sortfield->get_sort_field());
-            $sortcontentfull = $sortfield->get_sort_sql($sortcontent);
-
-            $what = ' DISTINCT r.id, r.approved, r.timecreated, r.timemodified, r.userid, ' . $namefields . ',
-                    ' . $sortcontentfull . ' AS sortorder ';
-            $count = ' COUNT(DISTINCT c.recordid) ';
-            $tables = '{data_content} c, {data_records} r, {user} u ';
-            $where =  'WHERE c.recordid = r.id
-                         AND r.dataid = :dataid
-                         AND r.userid = u.id ';
-            if (!$advanced) {
-                $where .=  'AND c.fieldid = :sort';
-            }
-            $params['dataid'] = $data->id;
-            $params['sort'] = $sort;
-            $sortorder = ' ORDER BY sortorder '.$order.' , r.id ASC ';
-            $searchselect = '';
-
-            // If requiredentries is not reached, only show current user's entries
-            if (!$requiredentries_allowed) {
-                $where .= ' AND u.id = :myid2';
-                $entrysql = ' AND r.userid = :myid3';
-                $params['myid2'] = $USER->id;
-                $initialparams['myid3'] = $params['myid2'];
-            }
-            $i = 0;
-            if (!empty($advanced)) {                                                  //If advanced box is checked.
-                foreach($search_array as $key => $val) {                              //what does $search_array hold?
-                    if ($key == DATA_FIRSTNAME or $key == DATA_LASTNAME) {
-                        $i++;
-                        $searchselect .= " AND ".$DB->sql_like($val->field, ":search_flname_$i", false);
-                        $params['search_flname_'.$i] = "%$val->data%";
-                        continue;
-                    }
-                    $advtables .= ', {data_content} c'.$key.' ';
-                    $advwhere .= ' AND c'.$key.'.recordid = r.id AND c'.$key.'.fieldid = '.$key;
-                    $advsearchselect .= ' AND ('.$val->sql.') ';
-                    $advparams = array_merge($advparams, $val->params);
-                }
-            } else if ($search) {
-                $searchselect = " AND (".$DB->sql_like('c.content', ':search1', false)." OR ".$DB->sql_like('u.firstname', ':search2', false)." OR ".$DB->sql_like('u.lastname', ':search3', false)." ) ";
-                $params['search1'] = "%$search%";
-                $params['search2'] = "%$search%";
-                $params['search3'] = "%$search%";
-            } else {
-                $searchselect = ' ';
-            }
-        }
-
-    /// To actually fetch the records
-
-        $fromsql    = "FROM $tables $advtables $where $advwhere $groupselect $approveselect $searchselect $advsearchselect";
-        $allparams  = array_merge($params, $advparams);
-
-        // Provide initial sql statements and parameters to reduce the number of total records.
-        $initialselect = $groupselect . $approveselect . $entrysql;
-
-        $recordids = data_get_all_recordids($data->id, $initialselect, $initialparams);
-        $newrecordids = data_get_advance_search_ids($recordids, $search_array, $data->id);
-        $totalcount = count($newrecordids);
-        $selectdata = $where . $groupselect . $approveselect;
-
-        if (!empty($advanced)) {
-            $advancedsearchsql = data_get_advanced_search_sql($sort, $data, $newrecordids, $selectdata, $sortorder);
-            $sqlselect = $advancedsearchsql['sql'];
-            $allparams = array_merge($allparams, $advancedsearchsql['params']);
-        } else {
-            $sqlselect  = "SELECT $what $fromsql $sortorder";
-        }
-
-        /// Work out the paging numbers and counts
-        if (empty($searchselect) && empty($advsearchselect)) {
-            $maxcount = $totalcount;
-        } else {
-            $maxcount = count($recordids);
-        }
-
-        if ($record) {     // We need to just show one, so where is it in context?
-            $nowperpage = 1;
-            $mode = 'single';
-            $page = 0;
-            // TODO MDL-33797 - Reduce this or consider redesigning the paging system.
-            if ($allrecordids = $DB->get_fieldset_sql($sqlselect, $allparams)) {
-                $page = (int)array_search($record->id, $allrecordids);
-                unset($allrecordids);
-            }
-        } else if ($mode == 'single') {  // We rely on ambient $page settings
-            $nowperpage = 1;
-
-        } else {
-            $nowperpage = $perpage;
-        }
-
-    /// Get the actual records
-
-        if (!$records = $DB->get_records_sql($sqlselect, $allparams, $page * $nowperpage, $nowperpage)) {
-            // Nothing to show!
-            if ($record) {         // Something was requested so try to show that at least (bug 5132)
-                if ($canmanageentries || empty($data->approval) ||
-                         $record->approved || (isloggedin() && $record->userid == $USER->id)) {
-                    if (!$currentgroup || $record->groupid == $currentgroup || $record->groupid == 0) {
-                        // OK, we can show this one
-                        $records = array($record->id => $record);
-                        $totalcount = 1;
-                    }
-                }
-            }
+        // Advanced search form doesn't make sense for single (redirects list view).
+        if ($maxcount && $mode != 'single') {
+            data_print_preference_form($data, $perpage, $search, $sort, $order, $search_array, $advanced, $mode);
         }
 
         if (empty($records)) {
@@ -722,7 +432,10 @@ if ($showactivity) {
                 echo $OUTPUT->notification(get_string('norecords','data'));
             }
 
-        } else { //  We have some records to print
+        } else {
+            //  We have some records to print.
+            $url = new moodle_url('/mod/data/view.php', array('d' => $data->id, 'sesskey' => sesskey()));
+            echo html_writer::start_tag('form', array('action' => $url, 'method' => 'post'));
 
             if ($maxcount != $totalcount) {
                 $a = new stdClass();
@@ -794,6 +507,31 @@ if ($showactivity) {
                 echo $OUTPUT->paging_bar($totalcount, $page, $nowperpage, $baseurl);
             }
 
+            if ($mode != 'single' && $canmanageentries) {
+                echo html_writer::empty_tag('input', array(
+                        'type' => 'button',
+                        'id' => 'checkall',
+                        'value' => get_string('selectall'),
+                        'class' => 'btn btn-secondary mr-1'
+                    ));
+                echo html_writer::empty_tag('input', array(
+                        'type' => 'button',
+                        'id' => 'checknone',
+                        'value' => get_string('deselectall'),
+                        'class' => 'btn btn-secondary mr-1'
+                    ));
+                echo html_writer::empty_tag('input', array(
+                        'class' => 'form-submit',
+                        'type' => 'submit',
+                        'value' => get_string('deleteselected'),
+                        'class' => 'btn btn-secondary mr-1'
+                    ));
+
+                $module = array('name' => 'mod_data', 'fullpath' => '/mod/data/module.js');
+                $PAGE->requires->js_init_call('M.mod_data.init_view', null, false, $module);
+            }
+
+            echo html_writer::end_tag('form');
         }
     }
 
@@ -802,29 +540,25 @@ if ($showactivity) {
         $records = array();
     }
 
-    if ($mode != 'single' && $canmanageentries) {
-        echo html_writer::empty_tag('input', array('type' => 'button', 'id' => 'checkall', 'value' => get_string('selectall')));
-        echo html_writer::empty_tag('input', array('type' => 'button', 'id' => 'checknone', 'value' => get_string('deselectall')));
-        echo html_writer::empty_tag('input', array('class' => 'form-submit', 'type' => 'submit', 'value' => get_string('deleteselected')));
-
-        $module = array('name'=>'mod_data', 'fullpath'=>'/mod/data/module.js');
-        $PAGE->requires->js_init_call('M.mod_data.init_view', null, false, $module);
-    }
-    echo html_writer::end_tag('form');
-
+    // Check to see if we can export records to a portfolio. This is for exporting all records, not just the ones in the search.
     if ($mode == '' && !empty($CFG->enableportfolios) && !empty($records)) {
-        require_once($CFG->libdir . '/portfoliolib.php');
-        $button = new portfolio_add_button();
-        $button->set_callback_options('data_portfolio_caller', array('id' => $cm->id), 'mod_data');
-        if (data_portfolio_caller::has_files($data)) {
-            $button->set_formats(array(PORTFOLIO_FORMAT_RICHHTML, PORTFOLIO_FORMAT_LEAP2A)); // no plain html for us
+        $canexport = false;
+        // Exportallentries and exportentry are basically the same capability.
+        if (has_capability('mod/data:exportallentries', $context) || has_capability('mod/data:exportentry', $context)) {
+            $canexport = true;
+        } else if (has_capability('mod/data:exportownentry', $context) &&
+                $DB->record_exists('data_records', array('userid' => $USER->id))) {
+            $canexport = true;
+        }
+        if ($canexport) {
+            require_once($CFG->libdir . '/portfoliolib.php');
+            $button = new portfolio_add_button();
+            $button->set_callback_options('data_portfolio_caller', array('id' => $cm->id), 'mod_data');
+            if (data_portfolio_caller::has_files($data)) {
+                $button->set_formats(array(PORTFOLIO_FORMAT_RICHHTML, PORTFOLIO_FORMAT_LEAP2A)); // No plain html for us.
+            }
+            echo $button->to_html(PORTFOLIO_ADD_FULL_FORM);
         }
-        echo $button->to_html(PORTFOLIO_ADD_FULL_FORM);
-    }
-
-    //Advanced search form doesn't make sense for single (redirects list view)
-    if (($maxcount || $mode == 'asearch') && $mode != 'single') {
-        data_print_preference_form($data, $perpage, $search, $sort, $order, $search_array, $advanced, $mode);
     }
 }
 
