@@ -18,7 +18,7 @@
 /**
  * This file adds support to rss feeds generation
  *
- * @package mod_forum
+ * @package   mod_forum
  * @category rss
  * @copyright 2001 Eloy Lafuente (stronk7) http://contiento.com
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
@@ -146,7 +146,7 @@ function forum_rss_feed_discussions_sql($forum, $cm, $newsince=0) {
 
     $modcontext = null;
 
-    $now = round(time(), -2);
+    $now = floor(time() / 60) * 60; // DB Cache Friendly.
     $params = array();
 
     $modcontext = context_module::instance($cm->id);
@@ -189,7 +189,7 @@ function forum_rss_feed_discussions_sql($forum, $cm, $newsince=0) {
               FROM {forum_discussions} d
                    JOIN {forum_posts} p ON p.discussion = d.id
                    JOIN {user} u ON p.userid = u.id
-             WHERE d.forum = {$forum->id} AND p.parent = 0
+             WHERE d.forum = {$forum->id} AND p.parent = 0 AND p.deleted <> 1
                    $timelimit $groupselect $newsince
           ORDER BY $forumsort";
     return array($sql, $params);
@@ -204,6 +204,8 @@ function forum_rss_feed_discussions_sql($forum, $cm, $newsince=0) {
  * @return string the SQL query to be used to get the Post details from the forum table of the database
  */
 function forum_rss_feed_posts_sql($forum, $cm, $newsince=0) {
+    global $USER;
+
     $modcontext = context_module::instance($cm->id);
 
     // Get group enforcement SQL.
@@ -224,23 +226,37 @@ function forum_rss_feed_posts_sql($forum, $cm, $newsince=0) {
         $newsince = '';
     }
 
+    $canseeprivatereplies = has_capability('mod/forum:readprivatereplies', $modcontext);
+    if (!$canseeprivatereplies) {
+        $privatewhere = ' AND (p.privatereplyto = :currentuser1 OR p.userid = :currentuser2 OR p.privatereplyto = 0)';
+        $params['currentuser1'] = $USER->id;
+        $params['currentuser2'] = $USER->id;
+    } else {
+        $privatewhere = '';
+    }
+
     $usernamefields = get_all_user_name_fields(true, 'u');
     $sql = "SELECT p.id AS postid,
                  d.id AS discussionid,
                  d.name AS discussionname,
+                 d.groupid,
+                 d.timestart,
+                 d.timeend,
                  u.id AS userid,
                  $usernamefields,
                  p.subject AS postsubject,
                  p.message AS postmessage,
                  p.created AS postcreated,
                  p.messageformat AS postformat,
-                 p.messagetrust AS posttrust
+                 p.messagetrust AS posttrust,
+                 p.parent as postparent
             FROM {forum_discussions} d,
                {forum_posts} p,
                {user} u
             WHERE d.forum = {$forum->id} AND
-                p.discussion = d.id AND
+                p.discussion = d.id AND p.deleted <> 1 AND
                 u.id = p.userid $newsince
+                $privatewhere
                 $groupselect
             ORDER BY p.created desc";
 
@@ -315,16 +331,37 @@ function forum_rss_feed_contents($forum, $sql, $params, $context) {
     foreach ($recs as $rec) {
             $item = new stdClass();
 
-            if ($isdiscussion && !forum_user_can_see_discussion($forum, $rec->discussionid, $context)) {
+            $discussion = new stdClass();
+            $discussion->id = $rec->discussionid;
+            $discussion->groupid = $rec->groupid;
+            $discussion->timestart = $rec->timestart;
+            $discussion->timeend = $rec->timeend;
+
+            $post = null;
+            if (!$isdiscussion) {
+                $post = new stdClass();
+                $post->id = $rec->postid;
+                $post->parent = $rec->postparent;
+                $post->userid = $rec->userid;
+            }
+
+            if ($isdiscussion && !forum_user_can_see_discussion($forum, $discussion, $context)) {
                 // This is a discussion which the user has no permission to view
                 $item->title = get_string('forumsubjecthidden', 'forum');
                 $message = get_string('forumbodyhidden', 'forum');
                 $item->author = get_string('forumauthorhidden', 'forum');
-            } else if (!$isdiscussion && !forum_user_can_see_post($forum, $rec->discussionid, $rec->postid, $USER, $cm)) {
-                // This is a post which the user has no permission to view
-                $item->title = get_string('forumsubjecthidden', 'forum');
-                $message = get_string('forumbodyhidden', 'forum');
-                $item->author = get_string('forumauthorhidden', 'forum');
+            } else if (!$isdiscussion && !forum_user_can_see_post($forum, $discussion, $post, $USER, $cm)) {
+                if (forum_user_can_see_post($forum, $discussion, $post, $USER, $cm, false)) {
+                    // This is a post which the user has no permission to view.
+                    $item->title = get_string('forumsubjecthidden', 'forum');
+                    $message = get_string('forumbodyhidden', 'forum');
+                    $item->author = get_string('forumauthorhidden', 'forum');
+                } else {
+                    // This is a post which has been deleted.
+                    $item->title = get_string('privacy:request:delete:post:subject', 'mod_forum');
+                    $message = get_string('privacy:request:delete:post:subject', 'mod_forum');
+                    $item->author = get_string('forumauthorhidden', 'forum');
+                }
             } else {
                 // The user must have permission to view
                 if ($isdiscussion && !empty($rec->discussionname)) {
