@@ -304,6 +304,9 @@ class question_type {
     public function save_question($question, $form) {
         global $USER, $DB, $OUTPUT;
 
+        // The actuall update/insert done with multiple DB access, so we do it in a transaction.
+        $transaction = $DB->start_delegated_transaction ();
+
         list($question->category) = explode(',', $form->category);
         $context = $this->get_context_by_category_id($question->category);
 
@@ -350,16 +353,22 @@ class question_type {
             $question->defaultmark = $form->defaultmark;
         }
 
-        if (isset($form->idnumber) && ((string) $form->idnumber !== '')) {
-            // While this check already exists in the form validation, this is a backstop preventing unnecessary errors.
-            if (strpos($form->category, ',') !== false) {
-                list($category, $categorycontextid) = explode(',', $form->category);
+        if (isset($form->idnumber)) {
+            if ((string) $form->idnumber === '') {
+                $question->idnumber = null;
             } else {
-                $category = $form->category;
-            }
-            if (!$DB->record_exists('question',
-                    ['idnumber' => $form->idnumber, 'category' => $category])) {
-                $question->idnumber = $form->idnumber;
+                // While this check already exists in the form validation,
+                // this is a backstop preventing unnecessary errors.
+                // Only set the idnumber if it has changed and will not cause a unique index violation.
+                if (strpos($form->category, ',') !== false) {
+                    list($category, $categorycontextid) = explode(',', $form->category);
+                } else {
+                    $category = $form->category;
+                }
+                if (!$DB->record_exists('question',
+                        ['idnumber' => $form->idnumber, 'category' => $category])) {
+                    $question->idnumber = $form->idnumber;
+                }
             }
         }
 
@@ -393,16 +402,6 @@ class question_type {
         }
         $DB->update_record('question', $question);
 
-        if ($newquestion) {
-            // Log the creation of this question.
-            $event = \core\event\question_created::create_from_question_instance($question, $context);
-            $event->trigger();
-        } else {
-            // Log the update of this question.
-            $event = \core\event\question_updated::create_from_question_instance($question, $context);
-            $event->trigger();
-        }
-
         // Now to save all the answers and type-specific options.
         $form->id = $question->id;
         $form->qtype = $question->qtype;
@@ -431,6 +430,18 @@ class question_type {
         $DB->set_field('question', 'version', question_hash($question),
                 array('id' => $question->id));
 
+        if ($newquestion) {
+            // Log the creation of this question.
+            $event = \core\event\question_created::create_from_question_instance($question, $context);
+            $event->trigger();
+        } else {
+            // Log the update of this question.
+            $event = \core\event\question_updated::create_from_question_instance($question, $context);
+            $event->trigger();
+        }
+
+        $transaction->allow_commit();
+
         return $question;
     }
 
